/* $Id$ */
/**************************************************************************
 *
 *  FILE            camera.c
 *  MODULE OF       vidgrabber
 *
 *  DESCRIPTION     
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdio.h>
#include <string.h>
#if defined(sgi)
#include <vl/vl.h>
#endif

#include <shhmsg.h>

#include "blockimg.h"
#include "camera.h"

#undef SAVEFILE "save.img"
#if !defined(sgi)
#define LOADFILE "capture.img"
#endif

/**************************************************************************
 *                                                                        *
 *                       P R I V A T E    D A T A                         *
 *                                                                        *
 **************************************************************************/

#if defined(LOADFILE)
static FILE *camFile;
static int camWidth = IMAGE_WIDTH;
#elif defined(sgi)
static char *camDisplayName = "";
static VLServer camServer;
static VLNode camSource, camDrain;
static VLPath camPath;
static VLBuffer camBuffer;
static int camWidth, camHeight;
#ifdef SAVEFILE
static FILE *camFile;
#endif
#endif



/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

static void
camFatal(void)
{
#if defined(sgi)
    msgFatal("camera error: %s\n", vlStrError(vlErrno));
#endif
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

void
camInit(void)
{
#if defined(sgi)
    VLControlValue cv;
#endif

#if defined(LOADFILE)
    if ((camFile = fopen(LOADFILE, "rb")) == NULL)
	msgFatalPerror(LOADFILE);
#elif defined(sgi)
    if ((camServer = vlOpenVideo(camDisplayName)) == NULL)
	camFatal();
    if ((camSource = vlGetNode(camServer, VL_SRC, VL_VIDEO, VL_ANY)) < 0)
	camFatal();
    if ((camDrain = vlGetNode(camServer, VL_DRN, VL_MEM, VL_ANY)) < 0)
	camFatal();
    if ((camPath = vlCreatePath(camServer, VL_ANY, camSource, camDrain)) < 0)
        camFatal();
    if (vlSetupPaths(camServer, (VLPathList) &camPath,
		     1, VL_LOCK, VL_LOCK) < 0)
        camFatal();
    cv.intVal = VL_PACKING_Y_8_P;
    if (vlSetControl(camServer, camPath, camDrain, VL_PACKING, &cv) < 0)
	camFatal();
    cv.fractVal.numerator = 1;
    cv.fractVal.denominator = 4;
    if (vlSetControl(camServer, camPath, camDrain, VL_ZOOM, &cv) < 0)
	camFatal();
    cv.xyVal.x = IMAGE_WIDTH;
    cv.xyVal.y = IMAGE_HEIGHT;
    if (vlSetControl(camServer, camPath, camDrain, VL_SIZE, &cv) < 0)
	camFatal();
    if ((camBuffer = vlCreateBuffer(camServer, camPath, camDrain, 1)) == NULL)
	camFatal();
    if (vlRegisterBuffer(camServer, camPath, camDrain, camBuffer) < 0)
	camFatal();
    vlGetControl(camServer, camPath, camDrain, VL_SIZE, &cv);
    camWidth = cv.xyVal.x;
    camHeight = cv.xyVal.y;
#ifdef SAVEFILE
    if ((camFile = fopen(SAVEFILE, "wb")) == NULL)
	msgFatalPerror(SAVEFILE);
#endif
#endif
}

void
camFinish(void)
{
#if defined(LOADFILE)
    fclose(camFile);
#elif defined(sgi)
    vlDeregisterBuffer(camServer, camPath, camDrain, camBuffer);
    vlDestroyBuffer(camServer, camBuffer);
    vlDestroyPath(camServer, camPath);
    vlCloseVideo(camServer);
#ifdef SAVEFILE
    fclose(camFile);
#endif
#endif
}

void
camGetFrame(BlockImage *blockImage)
{
#if defined(LOADFILE) || defined(sgi)
    int x, y, yb;
    register int xb;
    register unsigned char *d, *s;
    unsigned char tmpbuff[BLOCK_WIDTH * BLOCK_HEIGHT];
#endif
#if defined(LOADFILE)
    static unsigned char buffer[IMAGE_WIDTH * IMAGE_HEIGHT];
#elif defined(sgi)
    unsigned char *buffer;
    VLInfoPtr info;
    VLTransferDescriptor xferDesc;
#endif

#if defined(LOADFILE)
    if (fread(buffer, IMAGE_WIDTH * IMAGE_HEIGHT, 1, camFile) < 1) {
	rewind(camFile);
	fread(buffer, IMAGE_WIDTH * IMAGE_HEIGHT, 1, camFile);
    }
#elif defined(sgi)
    vlPutFree(camServer, camBuffer);
    xferDesc.mode = VL_TRANSFER_MODE_DISCRETE;
    xferDesc.count = 1;
    xferDesc.delay = 0;
    xferDesc.trigger = VLTriggerImmediate;
    if (vlBeginTransfer(camServer, camPath, 1, &xferDesc) == -1)
	camFatal();
    while ((info = vlGetLatestValid(camServer, camBuffer)) == NULL) {
	if (vlGetErrno())
	    camFatal();
    }
    vlEndTransfer(camServer, camPath);
    buffer = vlGetActiveRegion(camServer, camBuffer, info);
#ifdef SAVEFILE
    fwrite(buffer, IMAGE_WIDTH * IMAGE_HEIGHT, 1, camFile);
#endif
#endif
#if defined(LOADFILE) || defined(sgi)
    for (y = 0; y < VER_BLOCKS; y++)
	for (x = 0; x < HOR_BLOCKS; x++) {
	    d = tmpbuff;
	    for (yb = 0; yb < BLOCK_HEIGHT; yb++) {
		s = buffer + camWidth * (y * BLOCK_HEIGHT + yb)
		    + x * BLOCK_WIDTH;
		for (xb = BLOCK_WIDTH; xb; xb--)
#if 0
		    *d++ = (*s++ & 0xFF) >> (8 - GRAY_BITS);
#else
		    *d++ = *s++;
#endif
	    }
	    blkCalcMeasure(&(blockImage->block[x][y]), tmpbuff);
	    memcpy(blockImage->block[x][y].data, tmpbuff, sizeof(tmpbuff));
	}
#endif
}
