/* $Id$ */
/**************************************************************************
 *
 *  FILE            vidgrabber.c
 *  MODULE OF       vidgrabber
 *
 *  DESCRIPTION     
 *
 *  WRITTEN BY      Sverre H. Huseby <sverrehu@ifi.uio.no>
 *
 **************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include <shhmsg.h>
#include <shhopt.h>
#include <shhnet.h>

#include "camera.h"
#include "shhsched.h"
#include "network.h"
#include "blockimg.h"
#include "vidgrabber.h"

/**************************************************************************
 *                                                                        *
 *                       P R I V A T E    D A T A                         *
 *                                                                        *
 **************************************************************************/

static BlockImage blkImgOut;
static SchedId    grabSchedId;



/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

static void
vidGrabAndSend(void *clientData)
{
    grabSchedId = schedAddTime(SEND_DELAY, vidGrabAndSend, NULL);
    camGetFrame(&blkImgOut);
    netSendImage(&blkImgOut);
}

static void
version(void)
{
    printf(
      "%s " VERSION ", by Sverre H. Huseby "
      "(compiled " COMPILED_DATE " by " COMPILED_BY ")\n",
      msgGetName()
    );
    exit(0);
}

static void
usage(void)
{
    printf(
      "usage: %s [options] remote-host[:remote-port]\n"
      "\n"
      "  -h, --help      display this help and exit\n"
      "  -V, --version   output version information and exit\n"
      "\n",
      msgGetName()
    );
    exit(0);
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

void
vidStartGrab(void)
{
    grabSchedId = schedAddTime(SEND_DELAY, vidGrabAndSend, NULL);
}

void
vidStopGrab(void)
{
    schedRemoveTime(grabSchedId);
}

int
main(int argc, char *argv[])
{
    optStruct opt[] = {
      /* short long           type        var/func         special       */
        { 'h', "help",        OPT_FLAG,   usage,           OPT_CALLFUNC },
        { 'V', "version",     OPT_FLAG,   version,         OPT_CALLFUNC },
        { 0, 0, OPT_END, 0, 0 }  /* no more options */
    };

    msgSetName(argv[0]);
    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, msgFatal);

    optParseOptions(&argc, argv, opt, 0);
    if (argc != 2)
	usage();

    netSetRemoteSystem(argv[1]);

    schedInit();
    camInit();
    netInit();

    schedLoop();

    netFinish();
    camFinish();
    schedFinish();
    return 0;
}
